//
//  DigitalConfig.h
//  GJDigitalSDK
//
//  Created by ah on 2021/11/1.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

// Error code enumeration
typedef NS_ENUM(NSInteger, ErrorCode) {
    IM_CONNECT_ERROR,              // Signaling controller connection error, please check network connection status
    IM_LOGIN_ERROR,                // Signaling controller authentication failed
    IM_CONNECTION_CLOSED_ERROR,    // Signaling controller connection has been closed, unable to send signaling
    IM_RTC_REMOTE_SDP_FAILED,      // Failed to set remote SDP
    IM_RTC_LOCAL_SDP_FAILED,       // Failed to set local SDP
};


@interface DigitalConfig : NSObject

/**
 * Whether to open the microphone (default YES)
 */
@property (nonatomic, assign) BOOL isOpenMicrophone;

/**
 * openAsr (optional) YES: enable recognition, NO: disable recognition, default YES
 */
@property (nonatomic, assign) BOOL openAsr;

/**
 * Minimum bitrate, default 2000
 */
@property (nonatomic, strong) NSString *minBitrate;

/**
 * Maximum bitrate, default 4000
 */
@property (nonatomic, strong) NSString *maxBitrate;

/**
 * Reconnection time, default 0
 */
@property (nonatomic, assign) NSInteger timeToReconnected;

/**
 * Connection time, default 60000
 */
@property (nonatomic, assign) NSInteger timeToConnected;

/**
 * 0: remote video and audio, 1: remote audio only
 */
@property (nonatomic, assign) NSInteger rtc_audio_type;

/**
 * Whether to enable recording, default NO
 */
@property (nonatomic, assign) BOOL enableRec;

/**
 * One-shot recognition (not using audio/video): no, long link recognition: yes, default no
 */
@property (nonatomic, assign) BOOL isAsr;

/**
 * Whether to play local audio (default YES)
 */
@property (nonatomic, assign) BOOL isShowLocalAudio;

/**
 * Whether to show the local video frame (default NO)
 */
@property (nonatomic, assign) BOOL isShowLocalVideo;

/**
 * Whether to open the local camera (default NO)
 */
@property (nonatomic, assign) BOOL isOpenCamera;

/**
 * Whether to stop the local camera (default YES)
 */
@property (nonatomic, assign) BOOL isStopCamera;

/**
 * Front camera or rear camera, default front (yes: front, no: rear)
 */
@property (nonatomic, assign) BOOL usingFrontCamera;

/**
 * Digital human coordinates, position
 */
@property (nonatomic, strong) NSDictionary *humanConfig;

/**
 * Session ID
 */
@property (nonatomic, strong) NSString *sessionId;

/**
 * Platform userId (for troubleshooting)
 */
@property (nonatomic, strong) NSString *userId;

/**
 * Application name (for troubleshooting)
 */
@property (nonatomic, strong) NSString *appName;

/**
 * Extended field usage
 */
@property (nonatomic, strong) NSDictionary *extContext;

/**
 * Whether it is live broadcast, default NO
 */
@property (nonatomic, assign) BOOL isLive;

/**
 * Nation 0 China, 1 Overseas, read Resources.bundle configuration
 */
@property (nonatomic, assign) NSInteger nation;

/**
 * Main domain name, read Resources.bundle configuration
 */
@property (nonatomic, strong) NSString *baseUrl;


+ (DigitalConfig *)shareConfig;


@end

NS_ASSUME_NONNULL_END
