//
//  ViewController.m
//  DuixDemo
//
// Created by gj on 2023/12/4.
//

#import "ViewController.h"
#import <GJDigitalSDK/GJDigitalSDK.h>
#import "GJAccess.h"
#import "WJLocalNetWorkManager.h"
#include <ifaddrs.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>

//ConversationId
#define ConversationId @""
//APPID
#define AppId @""
//APPKEY
#define AppKey @""
//音频驱动音频地址示例
#define AudioUrl @"https://app.duix.ai/nfs/duix/tts-proxy/2024-07-04/5000-_-681987357994258432--D19B0EAA19E98BBB102F6421380085B7.wav"

@interface ViewController ()<DigitalViewDelegate>

// Custom view
@property (nonatomic, strong) UIView *customView;

// Whether to display
@property (nonatomic, assign) BOOL digitalShow;


@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.view.backgroundColor = [UIColor whiteColor];
    
    [self loadUI];
}

- (void)loadUI {
    
    self.customView=[[UIView alloc] initWithFrame:CGRectMake(0, 0, [[UIScreen mainScreen] bounds].size.width, [[UIScreen mainScreen] bounds].size.height)];
    [self.view addSubview:self.customView];
    
    UIButton *startbtn = [UIButton buttonWithType:UIButtonTypeCustom];
    startbtn.frame = CGRectMake(20, self.view.frame.size.height - 100 - 60, 100, 40);
    [startbtn setTitle:@"Start" forState:UIControlStateNormal];
    [startbtn setTitleColor:[UIColor orangeColor] forState:UIControlStateNormal];
    [startbtn addTarget:self action:@selector(toStart) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:startbtn];

    UIButton *stopbtn = [UIButton buttonWithType:UIButtonTypeCustom];
    stopbtn.frame = CGRectMake(self.view.frame.size.width - 120, self.view.frame.size.height - 100 - 60, 100, 40);
    [stopbtn setTitle:@"Stop" forState:UIControlStateNormal];
    [stopbtn setTitleColor:[UIColor orangeColor] forState:UIControlStateNormal];
    [stopbtn addTarget:self action:@selector(toStop) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:stopbtn];

    UIButton *breakbtn = [UIButton buttonWithType:UIButtonTypeCustom];
    breakbtn.frame = CGRectMake((self.view.frame.size.width - 120) / 2.0, self.view.frame.size.height - 100 - 60, 120, 40);
    [breakbtn setTitle:@"Interrupt" forState:UIControlStateNormal];
    [breakbtn setTitleColor:[UIColor orangeColor] forState:UIControlStateNormal];
    [breakbtn addTarget:self action:@selector(toBreak) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:breakbtn];

    UIButton *textbtn = [UIButton buttonWithType:UIButtonTypeCustom];
    textbtn.frame = CGRectMake(20, self.view.frame.size.height - 100, 100, 40);
    [textbtn setTitle:@"Text-driven" forState:UIControlStateNormal];
    [textbtn setTitleColor:[UIColor orangeColor] forState:UIControlStateNormal];
    [textbtn addTarget:self action:@selector(toTextPlay) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:textbtn];

    UIButton *audiobtn = [UIButton buttonWithType:UIButtonTypeCustom];
    audiobtn.frame = CGRectMake((self.view.frame.size.width - 120) / 2.0, self.view.frame.size.height - 100, 120, 40);
    [audiobtn setTitle:@"Audio-driven" forState:UIControlStateNormal];
    [audiobtn setTitleColor:[UIColor orangeColor] forState:UIControlStateNormal];
    [audiobtn addTarget:self action:@selector(toAudioPlay) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:audiobtn];

    UIButton *askbtn = [UIButton buttonWithType:UIButtonTypeCustom];
    askbtn.frame = CGRectMake(self.view.frame.size.width - 120, self.view.frame.size.height - 100, 100, 40);
    [askbtn setTitle:@"Text Q&A" forState:UIControlStateNormal];
    [askbtn setTitleColor:[UIColor orangeColor] forState:UIControlStateNormal];
    [askbtn addTarget:self action:@selector(toAskText) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:askbtn];
    
}

#pragma mark - UIButton Click Events

#pragma mark - Text Q&A
- (void)toAskText {
    // Command to ask a question
    [[DigitalManager manager] commandAskWithText:@"What is your name?" interrupt:YES];
}

#pragma mark - Audio-driven
- (void)toAudioPlay {
    // Command to play an audio event
    [[DigitalManager manager] commandEventWithAudioUrl:AudioUrl interrupt:YES];
}

#pragma mark - Text-driven
- (void)toTextPlay {
    // Command to send a text message
    [[DigitalManager manager] commandEventWithText:@"Can we chat?" interrupt:YES];
}

#pragma mark - Interrupt
- (void)toBreak {
    // Command to interrupt the digital assistant
    [[DigitalManager manager] toBreakDigital];
}

#pragma mark - Start Conversation
- (void)toStart {
    [DigitalManager manager].remote_view = self.customView; // Remote video view
    [DigitalManager manager].delegate = self; // Set delegate
    __weak typeof(self) weakSelf = self;
    triggerLocalNetworkPrivacyAlertObjC();
    [[GJAccess manager] getmicrophoneaccess:^(bool isPermis) {
        if (isPermis) {
            [weakSelf sdkInit]; // Initialize SDK if permission is granted
        }
    }];
}

#pragma mark - End Conversation
- (void)toStop {
    self.digitalShow = NO; // Hide digital display
    [[DigitalManager manager] toStop]; // Stop the session
}

#pragma mark - SDK Initialization
- (void)sdkInit {
    if (AppId.length == 0 || AppKey.length == 0 || ConversationId.length == 0) {
        NSLog(@"GJDigitalDemo==Please configure correct AppId, AppKey, ConversationId");
        return;
    }
    NSLog(@"GJDigitalDemo==Loading...");
    [[DigitalManager manager] initWithAppId:AppId appKey:AppKey conversationId:ConversationId block:^(BOOL isSuccee, NSString *errorMsg) {
        if (isSuccee) {
            [[DigitalManager manager] toStart]; // Start if initialized successfully
        } else {
            NSLog(@"GJDigitalDemo==errorMsg==%@", errorMsg);
        }
    }];
}

#pragma mark - SDK Delegate DigitalViewDelegate

#pragma mark - Video Load Completion
- (void)onVideoShow:(BOOL)isSuccess progress:(float)progress {
    if (isSuccess) {
        self.digitalShow = YES; // Show digital avatar if loaded successfully
        NSLog(@"GJDigitalDemo==Load Complete");
    } else {
        NSLog(@"GJDigitalDemo==Loading - %lf", progress); // Log loading progress
    }
}

#pragma mark - Error Handling
- (void)onError:(NSInteger)error_code errorMsg:(NSString *)errorMsg {
    // Handle different error codes
    switch (error_code) {
        case -1:
        case -2:
            // MQTT connection issues
            break;
        case 50001:
            // Invalid or empty AppId
            break;
        case 50002:
            // Resource check failed, contact admin
            break;
        case 50003:
            // Resource occupied, check and try again
            break;
        case 50004:
            // Human request timeout
            break;
        case 50005:
            // Exception retrieving resources from group
            break;
        case 50006:
            // Signature failure
            break;
        case 50007:
            // Insufficient total concurrency of resources, please check and try again
            break;
        case 50009:
            // Resource timeout or not configured
            break;
    }
    NSLog(@"GJDigitalDemo==errorMsg==%@", errorMsg);
}

#pragma mark - ASR Start Recognition
- (void)toWebrtcAsrStart {
    NSLog(@"GJDigitalDemo==ASR Start Recognition");
}

#pragma mark - ASR Recognition
- (void)toWebrtcAsrText:(NSString*)asrText isFinish:(BOOL)isFinish {
    NSLog(@"GJDigitalDemo==ASR Recognition==%@", asrText);
}

#pragma mark - Digital Avatar Starts Speaking
- (void)toSpeakStart:(NSDictionary *)dict {
    NSLog(@"GJDigitalDemo==Started Speaking - %@", dict);
}

#pragma mark - Digital Avatar Stops Speaking
- (void)toSpeakStop:(NSDictionary *)dict {
    NSLog(@"GJDigitalDemo==Stopped Speaking - %@", dict);
}

static NSArray<NSData *> * addressesOfDiscardServiceOnBroadcastCapableInterfaces(void) {
    struct ifaddrs * addrList = NULL;
    int err = getifaddrs(&addrList);
    if (err != 0) {
        return @[];
    }
    NSMutableArray<NSData *> * result = [NSMutableArray array];
    for (struct ifaddrs * cursor = addrList; cursor != NULL; cursor = cursor->ifa_next) {
        if ( (cursor->ifa_flags & IFF_BROADCAST) &&
            (cursor->ifa_addr != NULL)
            ) {
            switch (cursor->ifa_addr->sa_family) {
                case AF_INET: {
                    struct sockaddr_in sin = *(struct sockaddr_in *) cursor->ifa_addr;
                    sin.sin_port = htons(9);
                    NSData * addr = [NSData dataWithBytes:&sin length:sizeof(sin)];
                    [result addObject:addr];
                } break;
                case AF_INET6: {
                    struct sockaddr_in6 sin6 = *(struct sockaddr_in6 *) cursor->ifa_addr;
                    sin6.sin6_port = htons(9);
                    NSData * addr = [NSData dataWithBytes:&sin6 length:sizeof(sin6)];
                    [result addObject:addr];
                } break;
                default: {
                    // do nothing
                } break;
            }
        }
    }
    freeifaddrs(addrList);
    return result;
}

extern void triggerLocalNetworkPrivacyAlertObjC(void) {
    int sock4 = socket(AF_INET, SOCK_DGRAM, 0);
    int sock6 = socket(AF_INET6, SOCK_DGRAM, 0);
    
    if ((sock4 >= 0) && (sock6 >= 0)) {
        char message = '!';
        NSArray<NSData *> * addresses = addressesOfDiscardServiceOnBroadcastCapableInterfaces();
        for (NSData * address in addresses) {
            int sock = ((const struct sockaddr *) address.bytes)->sa_family == AF_INET ? sock4 : sock6;
            (void) sendto(sock, &message, sizeof(message), MSG_DONTWAIT, address.bytes, (socklen_t) address.length);
        }
    }
    close(sock4);
    close(sock6);
}

@end
